@@ -343,6 +343,32 @@ assert args[2] == 5
         self.assertEqual(self.dbus_props.GetAll("org.freedesktop.Test.Other"), {"color": "yellow"})
         self.assertEqual(self.dbus_props.Get("org.freedesktop.Test.Other", "color"), "yellow")
 
+        # add properties with complex types
+        self.dbus_mock.AddProperty("org.freedesktop.Test.Main", "intarray", dbus.Array([1, 2]))
+        self.dbus_mock.AddProperty("org.freedesktop.Test.Main", "vardict", dbus.Dictionary({"a": 1}))
+        self.dbus_mock.AddProperty("org.freedesktop.Test.Main", "struct", dbus.Struct((1, "a")))
+
+        changes = [
+            {
+                "version": 5,
+                "connected": False,
+            },
+            {
+                "intarray": [1, 3],
+            },
+            {
+                "vardict": {"b": "2"},
+            },
+            {
+                "struct": (2, "b"),
+            },
+            {
+                "intarray": [3, 7],
+                "vardict": {"d": dbus.Array([42, 1])},
+                "struct": (0, "test"),
+            },
+        ]
+
         changed_props = []
         ml = GLib.MainLoop()
 
@@ -357,28 +383,26 @@ assert args[2] == 5
             self.assertEqual(iface, "org.freedesktop.Test.Main")
 
             changed_props.append(changed)
-            ml.quit()
+
+            if len(changed_props) == len(changes):
+                ml.quit()
 
         match = self.dbus_con.add_signal_receiver(
             catch, interface_keyword="interface", path_keyword="path", member_keyword="member"
         )
 
         # change property using mock helper
-        self.dbus_mock.UpdateProperties(
-            "org.freedesktop.Test.Main",
-            {
-                "version": 5,
-                "connected": False,
-            },
-        )
+        for change in changes:
+            self.dbus_mock.UpdateProperties("org.freedesktop.Test.Main", change)
+            self.assertTrue(change.items() <= self.dbus_props.GetAll("org.freedesktop.Test.Main").items())
 
         GLib.timeout_add(3000, ml.quit)
         ml.run()
 
         match.remove()
 
-        self.assertEqual(self.dbus_props.GetAll("org.freedesktop.Test.Main"), {"version": 5, "connected": False})
-        self.assertEqual(changed_props, [{"version": 5, "connected": False}])
+        for i, change in enumerate(changes):
+            self.assertEqual(changed_props[i], change)
 
         # test adding properties with the array type
         self.dbus_mock.AddProperty("org.freedesktop.Test.Main", "array", dbus.Array(["first"], signature="s"))
@@ -923,6 +947,45 @@ def load(mock, parameters):
         self.assertIn('<node name="Thing1" />', xml)
         self.assertIn('<node name="Thing2" />', xml)
 
+    def test_add_object_with_subclass(self):
+        """Template with DBusMockObject subclass"""
+
+        with tempfile.NamedTemporaryFile(prefix="objmgr_", suffix=".py") as my_template:
+            my_template.write(
+                b"""
+import dbus
+import dbusmock
+BUS_NAME = 'org.test.Things'
+MAIN_OBJ = '/org/test/Things'
+MAIN_IFACE = 'org.test.Do'
+SYSTEM_BUS = False
+
+class Thing1(dbusmock.mockobject.DBusMockObject):
+    def __init__(self, *args, **kwargs):
+        super(Thing1, self).__init__(*args, **kwargs)
+        self.magic = kwargs.get('mock_data')
+
+    @dbus.service.method(MAIN_IFACE, in_signature='i', out_signature='i')
+    def Do0(self, input):
+        return self.magic + input
+
+def load(mock, parameters):
+    mock.AddObject('/org/test/Things/Thing1', MAIN_IFACE, {}, [],
+                   mock_class=Thing1,
+                   mock_data=42)
+"""
+            )
+            my_template.flush()
+            (p_mock, _) = self.spawn_server_template(my_template.name, stdout=subprocess.PIPE, system_bus=False)
+            self.addCleanup(p_mock.wait)
+            self.addCleanup(p_mock.terminate)
+            self.addCleanup(p_mock.stdout.close)
+
+        dbus_con = self.get_dbus(system_bus=False)
+        thing1 = dbus_con.get_object("org.test.Things", "/org/test/Things/Thing1")
+        self.assertEqual(thing1.Do0(0), 42)
+        self.assertEqual(thing1.Do0(1), 43)
+
     def test_reset(self):
         """Reset() puts the template back to pristine state"""
 
